/******************************************************************************
 *
 *
 * Project:  OpenCPN
 * Purpose:  GE2KAP Object
 * Author:   Paul Higgins
 *
 ***************************************************************************
 *   Copyright (C) 2010 by Paul Higgins                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,  USA.             *
 ***************************************************************************
 *
 */
#include "wx/wx.h"
#include "wx/tokenzr.h"
#include "wx/datetime.h"
#include "wx/sound.h"
#include <wx/wfstream.h>
#include <wx/dir.h>
#include <wx/filename.h>
#include <wx/debug.h>
#include <wx/graphics.h>

#include <stdlib.h>
#include <math.h>
#include <time.h>

#include "ge2kap_pi.h"
#include "gpxdocument.h"


//---------------------------------------------------------------------------------------
//          ge2kap Selector/Control Dialog Implementation
//---------------------------------------------------------------------------------------
IMPLEMENT_CLASS ( GE2KAPUIDialog, wxDialog )

BEGIN_EVENT_TABLE ( GE2KAPUIDialog, wxDialog )

            EVT_CLOSE ( GE2KAPUIDialog::OnClose )
            EVT_TEXT ( 200,GE2KAPUIDialog::On_Text_Changed )
END_EVENT_TABLE()


GE2KAPUIDialog::GE2KAPUIDialog( )
{
//      printf("GE2KAPUIDialog ctor\n");
      Init();
}

GE2KAPUIDialog::~GE2KAPUIDialog( )
{

}


void GE2KAPUIDialog::Init( )
{
   // wxLogMessage(_T("[GE2KAP] ge2kap Init "));
}


bool GE2KAPUIDialog::Create ( wxWindow *parent, ge2kap_pi *ppi, wxWindowID id,
                              const wxString& caption, const wxString initial_ge2kap_dir,
                              const wxPoint& pos, const wxSize& size, long style )
{
//      printf("GE2KAPUIDialog::Create\n");

      pParent = parent;
      pPlugIn = ppi;
      // char rxText[512];
      int count = 0;
      // char* pch;
      // int pch_offset;      

      m_currentge2kapDir = initial_ge2kap_dir;

      //    As a display optimization....
      //    if current color scheme is other than DAY,
      //    Then create the dialog ..WITHOUT.. borders and title bar.
      //    This way, any window decorations set by external themes, etc
      //    will not detract from night-vision

      long wstyle = wxDEFAULT_FRAME_STYLE;
//      if ( ( global_color_scheme != GLOBAL_COLOR_SCHEME_DAY ) && ( global_color_scheme != GLOBAL_COLOR_SCHEME_RGB ) )
//            wstyle |= ( wxNO_BORDER );

      wxSize size_min = size;
//      size_min.IncTo ( wxSize ( 500,600 ) );
      if ( !wxDialog::Create ( parent, id, caption, pos, size_min, wstyle ) )
            return false;


      // m_pfolder_bitmap = new wxBitmap ( folder );   // comes from XPM include

      CreateControls();

      DimeWindow(this);

      Fit();
      SetMinSize(GetBestSize());

      return true;
}

void GE2KAPUIDialog::CreateControls()
{
      int border_size = 4;
//      int check_spacing = 4;
      int group_item_spacing = 1;           // use for items within one group, with Add(...wxALL)


// A top-level sizer
      wxBoxSizer* topSizer = new wxBoxSizer ( wxVERTICAL );
      SetSizer ( topSizer );

// A second box sizer to give more space around the controls
      wxBoxSizer* boxSizer = new wxBoxSizer ( wxVERTICAL );
      topSizer->Add ( boxSizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALL|wxEXPAND, 2 );

//    The ge2kap directory
      wxStaticBox* itemStaticBoxSizer11Static = new wxStaticBox ( this, wxID_ANY,_ ( "ge2kap_plugin" ) );
      wxStaticBoxSizer *itemStaticBoxSizer11 = new wxStaticBoxSizer ( itemStaticBoxSizer11Static, wxHORIZONTAL );
      boxSizer->Add ( itemStaticBoxSizer11, 0, wxEXPAND );

      // m_pitemCurrentge2kapDirectoryCtrl = new wxTextCtrl (this, -1, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_READONLY );
      m_pitemCurrentge2kapDirectoryCtrl = new wxTextCtrl (this, 200, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_READONLY );
      itemStaticBoxSizer11->Add ( m_pitemCurrentge2kapDirectoryCtrl, 1, wxALIGN_LEFT|wxALL, 5 );

      m_pitemCurrentge2kapDirectoryCtrl->AppendText( m_currentge2kapDir );
}


void GE2KAPUIDialog::OnClose ( wxCloseEvent& event )
{
      // wxLogMessage(_T("[GE2KAP] gekap OnClose event"));

      pPlugIn->Onge2kapDialogClose();
}


void GE2KAPUIDialog::OnIdOKClick ( wxCommandEvent& event )
{
      Close();
}


void GE2KAPUIDialog::OnSize ( wxSizeEvent& event )
{
      //    Record the dialog size
      wxSize p = event.GetSize();
      pPlugIn->Setge2kapDialogSizeX(p.x);
      pPlugIn->Setge2kapDialogSizeY(p.y);

      event.Skip();
}

void GE2KAPUIDialog::On_Text_Changed( wxCommandEvent& event )
{    
   if (strstr(rxText,"gpx")!= NULL) // if its a valid gpx file
   {  
      wxLogMessage(_T("[GE2KAP] On_Text_Changed  lat_min=%f lat_max=%f lon_min=%f lon_max=%f"), my_vp.lat_min, my_vp.lat_max, my_vp.lon_min, my_vp.lon_max);
      // wxLogMessage(_T("[GE2KAP] On_Text_Changed  view_scale_ppm=%f m_projection_type=%d b_quilt=%d"), my_vp.view_scale_ppm,my_vp.m_projection_type,my_vp.b_quilt);
      // Get the lat/lon of the upperleft  and the lower right
      wxPoint p;
      p.x = 0;
      p.y = 0;
      double ullat,ullon;
      GetCanvasLLPix( &my_vp, p, &ullat, &ullon);
      p.x = my_vp.pix_width - 1;
      p.y = my_vp.pix_height - 1;
      double lrlat,lrlon;
      GetCanvasLLPix( &my_vp, p, &lrlat, &lrlon);
      wxString viewname;
      viewname.Printf(wxT("[GE2KAP] valid=%d skew=%f rotation=%f x=%d y=%d width=%d height=%d pix_width=%d pix_height=%d clat=%f clon=%f chart_scale=%f view_scale_ppm=%f m_projection_type=%d b_quilt=%d ullat=%f ullon=%f lrlat=%f lrlon=%f"), 
         my_vp.bValid,my_vp.skew,my_vp.rotation,my_vp.rv_rect.x, my_vp.rv_rect.y, my_vp.rv_rect.width, my_vp.rv_rect.height, my_vp.pix_width, my_vp.pix_height,
         my_vp.clat, my_vp.clon, my_vp.chart_scale, my_vp.view_scale_ppm, my_vp.m_projection_type, my_vp.b_quilt, ullat, ullon, lrlat, lrlon);
      wxLogMessage(viewname);       
      GpxDocument * gpx = new GpxDocument();
      GpxRootElement *gpxroot = (GpxRootElement *)gpx->RootElement();
      gpxroot->AddWaypoint(new GpxWptElement(GPX_WPT_WAYPOINT, my_vp.lat_max, my_vp.lon_min, 0, &wxDateTime::Now(), 0, -1, _T("UpperLeft"), wxEmptyString, viewname, wxEmptyString, NULL, _T("triangle"), _T("WPT")));
      gpxroot->AddWaypoint(new GpxWptElement(GPX_WPT_WAYPOINT, my_vp.lat_min, my_vp.lon_max, 0, &wxDateTime::Now(), 0, -1, _T("LowerRight"), wxEmptyString, viewname, wxEmptyString, NULL, _T("triangle"), _T("WPT")));
      gpxroot->AddWaypoint(new GpxWptElement(GPX_WPT_WAYPOINT,my_vp.lat_min, my_vp.lon_min, 0, &wxDateTime::Now(), 0, -1, _T("LowerLeft"), wxEmptyString, viewname, wxEmptyString, NULL, _T("triangle"), _T("WPT")));
      gpxroot->AddWaypoint(new GpxWptElement(GPX_WPT_WAYPOINT, my_vp.lat_max, my_vp.lon_max, 0, &wxDateTime::Now(), 0, -1, _T("UpperRight"), wxEmptyString, viewname, wxEmptyString, NULL, _T("triangle"), _T("WPT")));
      wxLogMessage(_T("[GE2KAP] On_Text_Changed saving gpx filename:%s"),m_gpx_filename);
      gpx->SaveFile(m_gpx_filename);
      gpx->Clear();
      delete gpx;  
   
   } 
}


WXLRESULT GE2KAPUIDialog::MSWDefWindowProc(WXUINT message, WXWPARAM wParam, WXLPARAM lParam) 
{ 
COPYDATASTRUCT* pMyCDS;
// const char* rxText;
/*
char rxText[512];
int count = 0;
char* pch;
int pch_offset;
*/
wxString copydata;
wxStringTokenizer tkz(copydata, _T(";"));
wxString          s_latitude;
wxString          s_longitude;
wxString          s_scale;
double latitude;
double longitude;
double scale;
// double scale = 65300.0;
 switch (message) 
 { 
   case WM_COPYDATA:
      pMyCDS = (PCOPYDATASTRUCT) lParam;
      memset(rxText,0,512);
   	dataln = pMyCDS->cbData; // get data length
   	memcpy(rxText,(const char *)(pMyCDS->lpData),dataln);   
      copydata = wxString::FromUTF8(rxText);   
      // wxStringTokenizer tkz(copydata, _T(";"));
      // int token_count = tkz.CountTokens();  // Make sure all four tokens are present
      tkz.SetString(copydata, _T(";"));
      if (tkz.CountTokens() == 4 )
      {
         s_latitude = tkz.GetNextToken();
         s_longitude = tkz.GetNextToken();
         s_scale = tkz.GetNextToken();
         m_gpx_filename = tkz.GetNextToken();
         wxLogMessage(_T("[GE2KAP] COPYDATA Latitude=%s Longitude=%s Scale=%s GPX_Filename=%s"),s_latitude, s_longitude, s_scale, m_gpx_filename);
         // if there is a lat/lon, jump to that position at the desrired scale
         if (s_latitude.IsEmpty() == false)
         {
            s_latitude.ToDouble(&latitude);  // convert to double 
            s_longitude.ToDouble(&longitude);  // convert to double  
            s_scale.ToDouble(&scale);  // convert to double  
            wxLogMessage(_T("[GE2KAP] COPYDATA Jumping to Latitude=%f Longitude=%f Scale=%f"),latitude, longitude, scale);  
            JumpToPosition(latitude, longitude, scale); 
            Sleep(1000); // sleep  1 second  
         }
         m_pitemCurrentge2kapDirectoryCtrl->SetValue (m_gpx_filename);  
      }  
      break;
   default: 
      break; 
 }
 return ::DefWindowProc(GetHwnd(), message, wParam, lParam);
} 

ge2kapPrefsDialog::ge2kapPrefsDialog( wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, long style ) : wxDialog( parent, id, title, pos, size, style )
{
	this->SetSizeHints( wxDefaultSize, wxDefaultSize );
	
	wxBoxSizer* bSizer2;
	bSizer2 = new wxBoxSizer( wxVERTICAL );
		
   m_cbShowIcon = new wxCheckBox( this, wxID_ANY, _("Show ge2kap icon"), wxDefaultPosition, wxDefaultSize, 0 );
   m_cbShowIcon->SetValue(true); 
	bSizer2->Add( m_cbShowIcon, 0, wxALL, 5 );
		
	m_sdbSizer1 = new wxStdDialogButtonSizer();
	m_sdbSizer1OK = new wxButton( this, wxID_OK );
	m_sdbSizer1->AddButton( m_sdbSizer1OK );
	m_sdbSizer1Cancel = new wxButton( this, wxID_CANCEL );
	m_sdbSizer1->AddButton( m_sdbSizer1Cancel );
	m_sdbSizer1->Realize();
	bSizer2->Add( m_sdbSizer1, 0, wxBOTTOM|wxEXPAND|wxTOP, 5 );
	
	this->SetSizer( bSizer2 );
	this->Layout();
	bSizer2->Fit( this );
	
	this->Centre( wxBOTH );
}

ge2kapPrefsDialog::~ge2kapPrefsDialog()
{
}
